const {
  createWorkflow,
  getWorkflowsByUserId,
  getWorkflowById,
  updateWorkflow,
  deleteWorkflow
} = require('../models/workflow');
const logger = require('../utils/logger');

/**
 * Create a new workflow
 * @param {Object} req - Express request object
 * @param {Object} res - Express response object
 * @param {Function} next - Express next middleware function
 */
const create = async (req, res, next) => {
  try {
    const { name, nodes, connections } = req.body;
    const userId = req.user.id;
    
    // Validate input
    if (!name) {
      return res.status(400).json({ 
        error: 'Bad Request', 
        message: 'Workflow name is required' 
      });
    }
    
    // Create workflow
    const workflow = await createWorkflow({ name, nodes, connections }, userId);
    
    res.status(201).json({
      message: 'Workflow created successfully',
      workflow
    });
  } catch (error) {
    next(error);
  }
};

/**
 * Get all workflows for the current user
 * @param {Object} req - Express request object
 * @param {Object} res - Express response object
 * @param {Function} next - Express next middleware function
 */
const getAll = async (req, res, next) => {
  try {
    const userId = req.user.id;
    
    // Get workflows
    const workflows = await getWorkflowsByUserId(userId);
    
    res.status(200).json({
      count: workflows.length,
      workflows
    });
  } catch (error) {
    next(error);
  }
};

/**
 * Get workflow by ID
 * @param {Object} req - Express request object
 * @param {Object} res - Express response object
 * @param {Function} next - Express next middleware function
 */
const getById = async (req, res, next) => {
  try {
    const { id } = req.params;
    const userId = req.user.id;
    
    // Get workflow
    const workflow = await getWorkflowById(id, userId);
    
    if (!workflow) {
      return res.status(404).json({ 
        error: 'Not Found', 
        message: 'Workflow not found' 
      });
    }
    
    res.status(200).json({ workflow });
  } catch (error) {
    next(error);
  }
};

/**
 * Update workflow
 * @param {Object} req - Express request object
 * @param {Object} res - Express response object
 * @param {Function} next - Express next middleware function
 */
const update = async (req, res, next) => {
  try {
    const { id } = req.params;
    const { name, nodes, connections } = req.body;
    const userId = req.user.id;
    
    // Update workflow
    const workflow = await updateWorkflow(id, { name, nodes, connections }, userId);
    
    if (!workflow) {
      return res.status(404).json({ 
        error: 'Not Found', 
        message: 'Workflow not found' 
      });
    }
    
    res.status(200).json({
      message: 'Workflow updated successfully',
      workflow
    });
  } catch (error) {
    next(error);
  }
};

/**
 * Delete workflow
 * @param {Object} req - Express request object
 * @param {Object} res - Express response object
 * @param {Function} next - Express next middleware function
 */
const remove = async (req, res, next) => {
  try {
    const { id } = req.params;
    const userId = req.user.id;
    
    // Delete workflow
    const deleted = await deleteWorkflow(id, userId);
    
    if (!deleted) {
      return res.status(404).json({ 
        error: 'Not Found', 
        message: 'Workflow not found' 
      });
    }
    
    res.status(200).json({
      message: 'Workflow deleted successfully'
    });
  } catch (error) {
    next(error);
  }
};

/**
 * Execute workflow
 * @param {Object} req - Express request object
 * @param {Object} res - Express response object
 * @param {Function} next - Express next middleware function
 */
const execute = async (req, res, next) => {
  try {
    const { id } = req.params;
    const userId = req.user.id;
    
    // Get workflow
    const workflow = await getWorkflowById(id, userId);
    
    if (!workflow) {
      return res.status(404).json({ 
        error: 'Not Found', 
        message: 'Workflow not found' 
      });
    }
    
    // TODO: Implement workflow execution logic with BullMQ
    // This will be implemented in the workflow execution service
    
    res.status(202).json({
      message: 'Workflow execution started',
      executionId: 'temp-id' // Will be replaced with actual execution ID
    });
  } catch (error) {
    next(error);
  }
};

module.exports = {
  create,
  getAll,
  getById,
  update,
  remove,
  execute
};
