const { db } = require('../config/db');
const { v4: uuidv4 } = require('uuid');
const bcrypt = require('bcrypt');

/**
 * Create a new user
 * @param {Object} userData - User data
 * @returns {Object} Created user
 */
const createUser = async (userData) => {
  // Hash password
  const salt = await bcrypt.genSalt(10);
  const hashedPassword = await bcrypt.hash(userData.password, salt);
  
  // Generate UUID for user
  const userId = uuidv4();
  
  // Insert user into database
  const [user] = await db('users')
    .insert({
      id: userId,
      email: userData.email.toLowerCase(),
      password: hashedPassword,
      created_at: new Date()
    })
    .returning(['id', 'email', 'created_at']);
  
  return user;
};

/**
 * Get user by email
 * @param {string} email - User email
 * @returns {Object|null} User object or null if not found
 */
const getUserByEmail = async (email) => {
  const user = await db('users')
    .where({ email: email.toLowerCase() })
    .first();
  
  return user;
};

/**
 * Get user by ID
 * @param {string} id - User ID
 * @returns {Object|null} User object or null if not found
 */
const getUserById = async (id) => {
  const user = await db('users')
    .where({ id })
    .first();
  
  return user;
};

/**
 * Verify user password
 * @param {string} password - Plain text password
 * @param {string} hashedPassword - Hashed password from database
 * @returns {boolean} True if password matches, false otherwise
 */
const verifyPassword = async (password, hashedPassword) => {
  return await bcrypt.compare(password, hashedPassword);
};

module.exports = {
  createUser,
  getUserByEmail,
  getUserById,
  verifyPassword
};
