const { db } = require('../config/db');
const { v4: uuidv4 } = require('uuid');

/**
 * Safely parse JSON data
 * @param {string|Object} data - Data to parse
 * @returns {Object} Parsed data
 */
const safeJsonParse = (data) => {
  if (!data) return [];
  if (typeof data === 'string') {
    try {
      return JSON.parse(data);
    } catch (error) {
      console.error('Error parsing JSON:', error);
      return [];
    }
  }
  return data; // Already an object
};

/**
 * Safely stringify JSON data
 * @param {string|Object} data - Data to stringify
 * @returns {string} Stringified data
 */
const safeJsonStringify = (data) => {
  if (!data) return '[]';
  if (typeof data === 'string') {
    return data; // Already a string
  }
  return JSON.stringify(data);
};

/**
 * Create a new workflow
 * @param {Object} workflowData - Workflow data
 * @param {string} userId - User ID
 * @returns {Object} Created workflow
 */
const createWorkflow = async (workflowData, userId) => {
  // Generate UUID for workflow
  const workflowId = uuidv4();
  
  // Safely stringify nodes and connections
  const nodesToStore = safeJsonStringify(workflowData.nodes || []);
  const connectionsToStore = safeJsonStringify(workflowData.connections || []);
    
  // Insert workflow into database
  const [workflow] = await db('workflows')
    .insert({
      id: workflowId,
      user_id: userId,
      name: workflowData.name,
      nodes: nodesToStore,
      connections: connectionsToStore,
      edges: connectionsToStore, // Also store in edges for backward compatibility
      created_at: new Date(),
      updated_at: new Date()
    })
    .returning(['id', 'user_id', 'name', 'nodes', 'connections', 'edges', 'created_at', 'updated_at']);
  
  // Safely parse JSON fields
  return {
    ...workflow,
    nodes: safeJsonParse(workflow.nodes),
    connections: safeJsonParse(workflow.connections || workflow.edges)
  };
};

/**
 * Get all workflows for a user
 * @param {string} userId - User ID
 * @returns {Array} Array of workflows
 */
const getWorkflowsByUserId = async (userId) => {
  const workflows = await db('workflows')
    .where({ user_id: userId })
    .select('*')
    .orderBy('updated_at', 'desc');
  
  // Safely parse JSON fields
  return workflows.map(workflow => ({
    ...workflow,
    nodes: safeJsonParse(workflow.nodes),
    connections: safeJsonParse(workflow.connections || workflow.edges)
  }));
};

/**
 * Get workflow by ID
 * @param {string} id - Workflow ID
 * @param {string} userId - User ID (for authorization)
 * @returns {Object|null} Workflow object or null if not found
 */
const getWorkflowById = async (id, userId) => {
  const workflow = await db('workflows')
    .where({ id, user_id: userId })
    .first();
  
  if (!workflow) {
    return null;
  }
  
  // Safely parse JSON fields
  // Use connections if available, otherwise fall back to edges
  const connections = workflow.connections ? safeJsonParse(workflow.connections) : safeJsonParse(workflow.edges);
  
  return {
    ...workflow,
    nodes: safeJsonParse(workflow.nodes),
    connections: connections
  };
};

/**
 * Update workflow
 * @param {string} id - Workflow ID
 * @param {Object} workflowData - Updated workflow data
 * @param {string} userId - User ID (for authorization)
 * @returns {Object|null} Updated workflow or null if not found
 */
const updateWorkflow = async (id, workflowData, userId) => {
  // Check if workflow exists and belongs to user
  const existingWorkflow = await getWorkflowById(id, userId);
  if (!existingWorkflow) {
    return null;
  }
  
  // Safely stringify nodes and connections
  const nodesToStore = safeJsonStringify(workflowData.nodes || existingWorkflow.nodes);
  const connectionsToStore = safeJsonStringify(workflowData.connections || existingWorkflow.connections);
    
  // Update workflow
  const [workflow] = await db('workflows')
    .where({ id, user_id: userId })
    .update({
      name: workflowData.name || existingWorkflow.name,
      nodes: nodesToStore,
      connections: connectionsToStore,
      edges: connectionsToStore, // Also update edges for backward compatibility
      updated_at: new Date()
    })
    .returning(['id', 'user_id', 'name', 'nodes', 'connections', 'edges', 'created_at', 'updated_at']);
  
  // Safely parse JSON fields
  return {
    ...workflow,
    nodes: safeJsonParse(workflow.nodes),
    connections: safeJsonParse(workflow.connections || workflow.edges)
  };
};

/**
 * Delete workflow
 * @param {string} id - Workflow ID
 * @param {string} userId - User ID (for authorization)
 * @returns {boolean} True if deleted, false if not found
 */
const deleteWorkflow = async (id, userId) => {
  // Check if workflow exists and belongs to user
  const existingWorkflow = await getWorkflowById(id, userId);
  if (!existingWorkflow) {
    return false;
  }
  
  // Delete workflow
  await db('workflows')
    .where({ id, user_id: userId })
    .delete();
  
  return true;
};

module.exports = {
  createWorkflow,
  getWorkflowsByUserId,
  getWorkflowById,
  updateWorkflow,
  deleteWorkflow
};
