const nodemailer = require('nodemailer');
const logger = require('../../utils/logger');

/**
 * Email Node Runner
 * Sends an email using SMTP
 */
async function run(nodeConfig, inputData) {
  try {
    logger.debug('Email node executing');
    
    // Check required configuration
    if (!nodeConfig.to || !nodeConfig.subject || !nodeConfig.body) {
      throw new Error('Missing required email configuration (to, subject, or body)');
    }
    
    if (!nodeConfig.smtpConfig || !nodeConfig.smtpConfig.host) {
      throw new Error('Missing SMTP configuration');
    }
    
    // Create transporter
    const transporter = nodemailer.createTransport({
      host: nodeConfig.smtpConfig.host,
      port: nodeConfig.smtpConfig.port || 587,
      secure: nodeConfig.smtpConfig.secure || false,
      auth: nodeConfig.smtpConfig.auth ? {
        user: nodeConfig.smtpConfig.auth.user,
        pass: nodeConfig.smtpConfig.auth.pass
      } : undefined
    });
    
    // Prepare email options
    const mailOptions = {
      from: nodeConfig.from,
      to: nodeConfig.to,
      subject: nodeConfig.subject,
      cc: nodeConfig.cc,
      bcc: nodeConfig.bcc
    };
    
    // Set email content based on HTML flag
    if (nodeConfig.isHtml) {
      mailOptions.html = nodeConfig.body;
    } else {
      mailOptions.text = nodeConfig.body;
    }
    
    // Send email
    const info = await transporter.sendMail(mailOptions);
    
    logger.debug('Email sent successfully', { 
      messageId: info.messageId,
      to: nodeConfig.to
    });
    
    // Return success response
    return {
      output: {
        success: true,
        messageId: info.messageId,
        response: info.response,
        timestamp: new Date().toISOString()
      }
    };
  } catch (error) {
    logger.error('Email node error', { error: error.message });
    
    // Return error through the error output
    return {
      error: {
        message: error.message,
        stack: error.stack
      }
    };
  }
}

module.exports = { run };
