const ivm = require('isolated-vm');
const logger = require('../../utils/logger');

/**
 * Function Node Runner
 * Executes custom JavaScript code in a sandboxed environment
 */
async function run(nodeConfig, inputData) {
  try {
    logger.debug('Function node executing');
    
    // Check required configuration
    if (!nodeConfig.code) {
      throw new Error('Missing function code');
    }
    
    // Set timeout (default: 5000ms)
    const timeout = nodeConfig.timeout || 5000;
    
    // Create a new isolated context
    const isolate = new ivm.Isolate({ memoryLimit: 32 });
    const context = await isolate.createContext();
    
    // Get a reference to the global object
    const jail = context.global;
    
    // Add input data to the context
    await jail.set('input', new ivm.ExternalCopy(inputData.input || {}).copyInto());
    
    // Add console methods that log to our logger
    const logMethods = {
      log: (...args) => logger.info('Function node log:', ...args),
      info: (...args) => logger.info('Function node info:', ...args),  
      warn: (...args) => logger.warn('Function node warn:', ...args),
      error: (...args) => logger.error('Function node error:', ...args)
    };
    
    await jail.set('console', new ivm.ExternalCopy(logMethods).copyInto());
    
    // Wrap the code in an async function that returns the result
    const wrappedCode = `
      (async function() {
        try {
          ${nodeConfig.code}
        } catch (error) {
          throw new Error(error.message);
        }
      })();
    `;
    
    // Execute the code with timeout
    const script = await isolate.compileScript(wrappedCode);
    const result = await script.run(context, { timeout });
    
    // Handle the result
    let output = {};
    if (result !== undefined && result !== null) {
      // If result is a primitive value, wrap it
      if (typeof result === 'object') {
        output = result;
      } else {
        output = { value: result };
      }
    }
    
    // Clean up
    isolate.dispose();
    
    return {
      output: output
    };
  } catch (error) {
    logger.error('Function node error', { error: error.message });
    
    // Return error through the error output
    return {
      error: {
        message: error.message,
        stack: error.stack
      }
    };
  }
}

module.exports = { run };
