const axios = require('axios');
const logger = require('../../utils/logger');

/**
 * HTTP Request Node Runner
 * Executes an HTTP request based on the node configuration
 */
async function run(nodeConfig, inputData) {
  try {
    logger.debug('HTTP Request node executing', { 
      url: nodeConfig.url,
      method: nodeConfig.method
    });

    // Build request config
    const requestConfig = {
      url: nodeConfig.url,
      method: nodeConfig.method || 'GET',
      timeout: nodeConfig.timeout || 5000
    };

    // Add headers if provided
    if (nodeConfig.headers && Object.keys(nodeConfig.headers).length > 0) {
      requestConfig.headers = nodeConfig.headers;
    }

    // Add request body for POST, PUT, PATCH methods
    if (['POST', 'PUT', 'PATCH'].includes(requestConfig.method) && nodeConfig.body) {
      try {
        // If body is a string that looks like JSON, parse it
        if (typeof nodeConfig.body === 'string') {
          requestConfig.data = JSON.parse(nodeConfig.body);
        } else {
          requestConfig.data = nodeConfig.body;
        }
      } catch (error) {
        // If parsing fails, use as is
        requestConfig.data = nodeConfig.body;
      }
    }

    // Execute the HTTP request
    const response = await axios(requestConfig);

    // Return the response data
    return {
      output: {
        status: response.status,
        statusText: response.statusText,
        headers: response.headers,
        data: response.data
      }
    };
  } catch (error) {
    logger.error('HTTP Request node error', { 
      error: error.message,
      url: nodeConfig.url
    });

    // Format axios error response
    const errorResponse = {
      message: error.message,
      code: error.code
    };

    // Add response data if available
    if (error.response) {
      errorResponse.status = error.response.status;
      errorResponse.statusText = error.response.statusText;
      errorResponse.headers = error.response.headers;
      errorResponse.data = error.response.data;
    }

    // Return error through the error output
    return {
      error: errorResponse
    };
  }
}

module.exports = { run };
