const fs = require('fs');
const path = require('path');
const logger = require('../utils/logger');

class NodeRegistry {
  constructor() {
    this.nodes = new Map();
    this.nodesPath = path.join(__dirname, '../nodes');
  }

  /**
   * Initialize the node registry by loading all available nodes
   */
  async initialize() {
    try {
      logger.info('Initializing node registry');
      
      // Get all node type directories
      const nodeTypes = fs.readdirSync(this.nodesPath, { withFileTypes: true })
        .filter(dirent => dirent.isDirectory())
        .map(dirent => dirent.name);
      
      // Load each node type
      for (const nodeType of nodeTypes) {
        await this.loadNodeType(nodeType);
      }
      
      logger.info(`Node registry initialized with ${this.nodes.size} node types`);
      return true;
    } catch (error) {
      logger.error('Failed to initialize node registry', { error: error.message });
      return false;
    }
  }

  /**
   * Load a specific node type
   * @param {string} nodeType - The node type to load
   */
  async loadNodeType(nodeType) {
    try {
      const nodePath = path.join(this.nodesPath, nodeType);
      
      // Load meta.json
      const metaPath = path.join(nodePath, 'meta.json');
      if (!fs.existsSync(metaPath)) {
        logger.warn(`Node type ${nodeType} missing meta.json, skipping`);
        return;
      }
      
      const meta = JSON.parse(fs.readFileSync(metaPath, 'utf8'));
      
      // Load runner.js
      const runnerPath = path.join(nodePath, 'runner.js');
      if (!fs.existsSync(runnerPath)) {
        logger.warn(`Node type ${nodeType} missing runner.js, skipping`);
        return;
      }
      
      const runner = require(runnerPath);
      
      // Register the node type
      this.nodes.set(nodeType, {
        meta,
        runner
      });
      
      logger.debug(`Loaded node type: ${nodeType}`);
    } catch (error) {
      logger.error(`Failed to load node type: ${nodeType}`, { error: error.message });
    }
  }

  /**
   * Get all registered node types
   * @returns {Array} Array of node type metadata
   */
  getAllNodeTypes() {
    return Array.from(this.nodes.values()).map(node => node.meta);
  }

  /**
   * Get a specific node type by type
   * @param {string} nodeType - The node type to get
   * @returns {Object|null} Node type or null if not found
   */
  getNodeType(nodeType) {
    return this.nodes.get(nodeType) || null;
  }

  /**
   * Execute a node
   * @param {string} nodeType - The node type to execute
   * @param {Object} nodeConfig - Node configuration
   * @param {Object} inputData - Input data for the node
   * @returns {Promise<Object>} Node execution result
   */
  async executeNode(nodeType, nodeConfig, inputData) {
    const node = this.getNodeType(nodeType);
    
    if (!node) {
      throw new Error(`Node type not found: ${nodeType}`);
    }
    
    if (!node.runner.run) {
      throw new Error(`Node type ${nodeType} does not have a run method`);
    }
    
    return await node.runner.run(nodeConfig, inputData);
  }
}

// Create and export a singleton instance
const nodeRegistry = new NodeRegistry();

module.exports = nodeRegistry;
