const Bull = require('bull');
const { v4: uuidv4 } = require('uuid');
const { db } = require('../config/db');
const nodeRegistry = require('./nodeRegistry');
const logger = require('../utils/logger');

class WorkflowExecutor {
  constructor() {
    // Initialize the workflow queue
    this.workflowQueue = new Bull('workflow-execution', process.env.REDIS_URL);
    
    // Initialize the execution logs map
    this.executionLogs = new Map();
    
    // Set up queue processing
    this.setupQueueProcessor();
  }

  /**
   * Set up the queue processor
   */
  setupQueueProcessor() {
    this.workflowQueue.process(async (job) => {
      const { workflowId, executionId, triggerNodeId, triggerData } = job.data;
      return await this.processWorkflow(workflowId, executionId, triggerNodeId, triggerData);
    });
    
    // Handle completed jobs
    this.workflowQueue.on('completed', (job, result) => {
      logger.info(`Workflow execution completed: ${job.data.executionId}`, { 
        workflowId: job.data.workflowId,
        success: result.success
      });
    });
    
    // Handle failed jobs
    this.workflowQueue.on('failed', (job, error) => {
      logger.error(`Workflow execution failed: ${job.data.executionId}`, { 
        workflowId: job.data.workflowId,
        error: error.message
      });
    });
  }

  /**
   * Execute a workflow
   * @param {string} workflowId - The workflow ID
   * @param {string} triggerNodeId - The ID of the trigger node
   * @param {Object} triggerData - Data from the trigger
   * @returns {Object} Execution details
   */
  async executeWorkflow(workflowId, triggerNodeId, triggerData) {
    // Generate a unique execution ID
    const executionId = uuidv4();
    
    // Initialize execution logs
    this.executionLogs.set(executionId, []);
    
    // Add the job to the queue
    await this.workflowQueue.add({
      workflowId,
      executionId,
      triggerNodeId,
      triggerData
    });
    
    logger.info(`Workflow execution queued: ${executionId}`, { workflowId });
    
    return {
      executionId,
      status: 'queued',
      timestamp: new Date().toISOString()
    };
  }

  /**
   * Process a workflow execution
   * @param {string} workflowId - The workflow ID
   * @param {string} executionId - The execution ID
   * @param {string} triggerNodeId - The ID of the trigger node
   * @param {Object} triggerData - Data from the trigger
   * @returns {Object} Execution result
   */
  async processWorkflow(workflowId, executionId, triggerNodeId, triggerData) {
    try {
      // Log execution start
      this.logExecution(executionId, 'info', 'Workflow execution started', {
        workflowId,
        triggerNodeId
      });
      
      // Get the workflow from the database
      const workflow = await db('workflows')
        .where({ id: workflowId })
        .first();
      
      if (!workflow) {
        throw new Error(`Workflow not found: ${workflowId}`);
      }
      
      // Parse JSON fields
      const nodes = JSON.parse(workflow.nodes);
      const connections = JSON.parse(workflow.connections);
      
      // Find the trigger node
      const triggerNode = nodes.find(node => node.id === triggerNodeId);
      if (!triggerNode) {
        throw new Error(`Trigger node not found: ${triggerNodeId}`);
      }
      
      // Execute the workflow starting from the trigger node
      const result = await this.executeNode(
        executionId,
        triggerNode,
        nodes,
        connections,
        triggerData
      );
      
      // Log execution completion
      this.logExecution(executionId, 'info', 'Workflow execution completed', {
        workflowId,
        success: true
      });
      
      // Store execution logs in the database
      await this.saveExecutionLogs(workflowId, executionId);
      
      return {
        success: true,
        executionId,
        result
      };
    } catch (error) {
      // Log execution error
      this.logExecution(executionId, 'error', 'Workflow execution failed', {
        workflowId,
        error: error.message
      });
      
      // Store execution logs in the database
      await this.saveExecutionLogs(workflowId, executionId);
      
      return {
        success: false,
        executionId,
        error: error.message
      };
    }
  }

  /**
   * Execute a node and follow connections to next nodes
   * @param {string} executionId - The execution ID
   * @param {Object} node - The node to execute
   * @param {Array} allNodes - All nodes in the workflow
   * @param {Array} connections - All connections in the workflow
   * @param {Object} inputData - Input data for the node
   * @returns {Object} Node execution results
   */
  async executeNode(executionId, node, allNodes, connections, inputData) {
    try {
      // Log node execution start
      this.logExecution(executionId, 'debug', `Executing node: ${node.id}`, {
        nodeType: node.type,
        nodeId: node.id
      });
      
      // Execute the node
      const nodeResult = await nodeRegistry.executeNode(
        node.type,
        node.config || {},
        { input: inputData }
      );
      
      // Log node execution result
      this.logExecution(executionId, 'debug', `Node execution result: ${node.id}`, {
        nodeId: node.id,
        outputs: Object.keys(nodeResult)
      });
      
      // Find connections from this node
      const nodeConnections = connections.filter(conn => conn.from === node.id);
      
      // Execute connected nodes
      const nextResults = {};
      
      for (const conn of nodeConnections) {
        const nextNode = allNodes.find(n => n.id === conn.to);
        
        if (nextNode) {
          // Get the output data from the current node
          const outputData = nodeResult.output || {};
          
          // Execute the next node
          nextResults[nextNode.id] = await this.executeNode(
            executionId,
            nextNode,
            allNodes,
            connections,
            outputData
          );
        }
      }
      
      // Return combined results
      return {
        nodeId: node.id,
        nodeType: node.type,
        result: nodeResult,
        nextNodes: nextResults
      };
    } catch (error) {
      // Log node execution error
      this.logExecution(executionId, 'error', `Node execution error: ${node.id}`, {
        nodeId: node.id,
        error: error.message
      });
      
      throw error;
    }
  }

  /**
   * Log an execution event
   * @param {string} executionId - The execution ID
   * @param {string} level - Log level (debug, info, warn, error)
   * @param {string} message - Log message
   * @param {Object} data - Additional log data
   */
  logExecution(executionId, level, message, data = {}) {
    // Create log entry
    const logEntry = {
      timestamp: new Date().toISOString(),
      level,
      message,
      data
    };
    
    // Add to execution logs
    if (this.executionLogs.has(executionId)) {
      this.executionLogs.get(executionId).push(logEntry);
    } else {
      this.executionLogs.set(executionId, [logEntry]);
    }
    
    // Also log to system logger
    logger[level](message, { executionId, ...data });
  }

  /**
   * Save execution logs to the database
   * @param {string} workflowId - The workflow ID
   * @param {string} executionId - The execution ID
   */
  async saveExecutionLogs(workflowId, executionId) {
    try {
      // Get logs for this execution
      const logs = this.executionLogs.get(executionId) || [];
      
      // Insert into database
      await db('workflow_logs').insert({
        id: executionId,
        workflow_id: workflowId,
        logs: JSON.stringify(logs),
        created_at: new Date()
      });
      
      // Clear logs from memory
      this.executionLogs.delete(executionId);
    } catch (error) {
      logger.error('Failed to save execution logs', { 
        executionId,
        error: error.message
      });
    }
  }

  /**
   * Get execution logs
   * @param {string} executionId - The execution ID
   * @returns {Array} Execution logs
   */
  async getExecutionLogs(executionId) {
    // Check if logs are in memory
    if (this.executionLogs.has(executionId)) {
      return this.executionLogs.get(executionId);
    }
    
    // Get logs from database
    const logRecord = await db('workflow_logs')
      .where({ id: executionId })
      .first();
    
    if (!logRecord) {
      return [];
    }
    
    return JSON.parse(logRecord.logs);
  }
}

// Create and export a singleton instance
const workflowExecutor = new WorkflowExecutor();

module.exports = workflowExecutor;
