import React, { useState, useEffect } from 'react';
import { toast } from 'react-toastify';
import api from '../../services/api';
import { XMarkIcon, ChevronDownIcon, ChevronUpIcon } from '@heroicons/react/24/outline';

const ExecutionResults = ({ workflowId, executionId, onClose }) => {
  const [execution, setExecution] = useState(null);
  const [logs, setLogs] = useState([]);
  const [loading, setLoading] = useState(true);
  const [expandedNodes, setExpandedNodes] = useState({});

  useEffect(() => {
    const fetchExecutionResults = async () => {
      try {
        setLoading(true);
        
        // Fetch execution details
        const executionResponse = await api.get(`/api/workflows/${workflowId}/executions/${executionId}`)
          .catch(err => {
            // If execution not found, handle gracefully
            if (err.response && err.response.status === 404) {
              console.log('Execution not found or still initializing');
              return { data: { execution: { status: 'initializing' } } };
            }
            throw err;
          });
        
        setExecution(executionResponse.data.execution);
        
        // Only fetch logs if execution exists and is not initializing
        if (executionResponse.data.execution && executionResponse.data.execution.status !== 'initializing') {
          const logsResponse = await api.get(`/api/workflows/${workflowId}/executions/${executionId}/logs`)
            .catch(err => {
              // If logs not found, handle gracefully
              if (err.response && err.response.status === 404) {
                return { data: { logs: [] } };
              }
              throw err;
            });
          
          setLogs(logsResponse.data.logs || []);
        }
      } catch (error) {
        console.error('Error fetching execution results:', error);
        toast.error('Failed to load execution results');
      } finally {
        setLoading(false);
      }
    };
    
    if (workflowId && executionId) {
      fetchExecutionResults();
    }
  }, [workflowId, executionId]);

  // Toggle expanded state for a node
  const toggleNodeExpanded = (nodeId) => {
    setExpandedNodes(prev => ({
      ...prev,
      [nodeId]: !prev[nodeId]
    }));
  };

  // Format timestamp
  const formatTimestamp = (timestamp) => {
    if (!timestamp) return '';
    const date = new Date(timestamp);
    return date.toLocaleString();
  };

  // Format duration
  const formatDuration = (startTime, endTime) => {
    if (!startTime || !endTime) return '';
    const start = new Date(startTime);
    const end = new Date(endTime);
    const durationMs = end - start;
    
    if (durationMs < 1000) {
      return `${durationMs}ms`;
    } else if (durationMs < 60000) {
      return `${(durationMs / 1000).toFixed(2)}s`;
    } else {
      const minutes = Math.floor(durationMs / 60000);
      const seconds = ((durationMs % 60000) / 1000).toFixed(0);
      return `${minutes}m ${seconds}s`;
    }
  };

  // Get status badge
  const getStatusBadge = (status) => {
    switch (status) {
      case 'success':
        return (
          <span className="inline-flex items-center px-2.5 py-0.5 rounded-full text-xs font-medium bg-green-100 text-green-800">
            Success
          </span>
        );
      case 'error':
        return (
          <span className="inline-flex items-center px-2.5 py-0.5 rounded-full text-xs font-medium bg-red-100 text-red-800">
            Error
          </span>
        );
      case 'running':
        return (
          <span className="inline-flex items-center px-2.5 py-0.5 rounded-full text-xs font-medium bg-blue-100 text-blue-800">
            Running
          </span>
        );
      case 'initializing':
        return (
          <span className="inline-flex items-center px-2.5 py-0.5 rounded-full text-xs font-medium bg-yellow-100 text-yellow-800">
            Initializing
          </span>
        );
      default:
        return (
          <span className="inline-flex items-center px-2.5 py-0.5 rounded-full text-xs font-medium bg-gray-100 text-gray-800">
            {status}
          </span>
        );
    }
  };

  return (
    <div className="bg-white shadow-xl rounded-lg overflow-hidden max-w-4xl w-full max-h-[80vh] flex flex-col">
      {/* Header */}
      <div className="px-4 py-5 sm:px-6 border-b border-gray-200 flex justify-between items-center">
        <h3 className="text-lg leading-6 font-medium text-gray-900">
          Execution Results
        </h3>
        <button
          onClick={onClose}
          className="rounded-md text-gray-400 hover:text-gray-500 focus:outline-none focus:ring-2 focus:ring-primary-500"
        >
          <XMarkIcon className="h-5 w-5" />
        </button>
      </div>
      
      {/* Content */}
      <div className="flex-1 overflow-y-auto">
        {loading ? (
          <div className="p-6 flex items-center justify-center">
            <div className="animate-spin rounded-full h-12 w-12 border-t-2 border-b-2 border-primary-500"></div>
          </div>
        ) : (
          <div className="divide-y divide-gray-200">
            {/* Execution summary */}
            {execution && (
              <div className="px-4 py-5 sm:p-6">
                <dl className="grid grid-cols-1 gap-x-4 gap-y-6 sm:grid-cols-2">
                  <div>
                    <dt className="text-sm font-medium text-gray-500">Status</dt>
                    <dd className="mt-1">{getStatusBadge(execution.status)}</dd>
                  </div>
                  <div>
                    <dt className="text-sm font-medium text-gray-500">Started At</dt>
                    <dd className="mt-1 text-sm text-gray-900">{formatTimestamp(execution.started_at)}</dd>
                  </div>
                  <div>
                    <dt className="text-sm font-medium text-gray-500">Completed At</dt>
                    <dd className="mt-1 text-sm text-gray-900">{formatTimestamp(execution.completed_at)}</dd>
                  </div>
                  <div>
                    <dt className="text-sm font-medium text-gray-500">Duration</dt>
                    <dd className="mt-1 text-sm text-gray-900">
                      {formatDuration(execution.started_at, execution.completed_at)}
                    </dd>
                  </div>
                  {execution.error && (
                    <div className="sm:col-span-2">
                      <dt className="text-sm font-medium text-gray-500">Error</dt>
                      <dd className="mt-1 text-sm text-red-600 bg-red-50 p-3 rounded">
                        {execution.error}
                      </dd>
                    </div>
                  )}
                </dl>
              </div>
            )}
            
            {/* Node execution logs */}
            <div className="px-4 py-5 sm:p-6">
              <h4 className="text-base font-medium text-gray-900 mb-4">Node Execution Logs</h4>
              
              {logs.length > 0 ? (
                <div className="space-y-4">
                  {logs.map((log) => (
                    <div key={log.id} className="border border-gray-200 rounded-md overflow-hidden">
                      <div 
                        className="bg-gray-50 px-4 py-3 flex justify-between items-center cursor-pointer"
                        onClick={() => toggleNodeExpanded(log.id)}
                      >
                        <div className="flex items-center">
                          <div className="font-medium text-gray-900">{log.node_name}</div>
                          <div className="ml-2 text-xs text-gray-500">({log.node_type})</div>
                          <div className="ml-3">{getStatusBadge(log.status)}</div>
                        </div>
                        <div className="flex items-center">
                          <div className="text-xs text-gray-500 mr-2">
                            {formatDuration(log.started_at, log.completed_at)}
                          </div>
                          {expandedNodes[log.id] ? (
                            <ChevronUpIcon className="h-5 w-5 text-gray-400" />
                          ) : (
                            <ChevronDownIcon className="h-5 w-5 text-gray-400" />
                          )}
                        </div>
                      </div>
                      
                      {expandedNodes[log.id] && (
                        <div className="px-4 py-3 border-t border-gray-200">
                          <div className="grid grid-cols-1 gap-4 sm:grid-cols-2">
                            <div>
                              <h5 className="text-sm font-medium text-gray-700 mb-1">Input</h5>
                              <pre className="text-xs bg-gray-50 p-2 rounded overflow-x-auto">
                                {JSON.stringify(log.input_data, null, 2)}
                              </pre>
                            </div>
                            <div>
                              <h5 className="text-sm font-medium text-gray-700 mb-1">Output</h5>
                              <pre className="text-xs bg-gray-50 p-2 rounded overflow-x-auto">
                                {JSON.stringify(log.output_data, null, 2)}
                              </pre>
                            </div>
                          </div>
                          
                          {log.error && (
                            <div className="mt-3">
                              <h5 className="text-sm font-medium text-red-700 mb-1">Error</h5>
                              <pre className="text-xs bg-red-50 text-red-700 p-2 rounded overflow-x-auto">
                                {log.error}
                              </pre>
                            </div>
                          )}
                        </div>
                      )}
                    </div>
                  ))}
                </div>
              ) : (
                <p className="text-gray-500 text-sm">No execution logs available</p>
              )}
            </div>
          </div>
        )}
      </div>
      
      {/* Footer */}
      <div className="px-4 py-3 bg-gray-50 text-right sm:px-6">
        <button
          onClick={onClose}
          className="inline-flex justify-center py-2 px-4 border border-transparent shadow-sm text-sm font-medium rounded-md text-white bg-primary-600 hover:bg-primary-700 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-primary-500"
        >
          Close
        </button>
      </div>
    </div>
  );
};

export default ExecutionResults;
