import React, { useState, useEffect } from 'react';
import { toast } from 'react-toastify';
import api from '../../services/api';
import {
  CalendarIcon,
  ClockIcon,
  PlusIcon,
  TrashIcon
} from '@heroicons/react/24/outline';

const CronScheduler = ({ workflowId }) => {
  const [schedules, setSchedules] = useState([]);
  const [loading, setLoading] = useState(true);
  const [showAddForm, setShowAddForm] = useState(false);
  const [newSchedule, setNewSchedule] = useState({
    cron: '0 0 * * *', // Default: daily at midnight
    enabled: true,
    name: ''
  });

  // Predefined cron expressions
  const cronPresets = [
    { label: 'Every minute', value: '* * * * *' },
    { label: 'Every hour', value: '0 * * * *' },
    { label: 'Every day at midnight', value: '0 0 * * *' },
    { label: 'Every Monday at 9:00 AM', value: '0 9 * * 1' },
    { label: 'Every month on the 1st at midnight', value: '0 0 1 * *' }
  ];

  // Fetch schedules
  useEffect(() => {
    const fetchSchedules = async () => {
      try {
        setLoading(true);
        const response = await api.get(`/api/workflows/${workflowId}/schedules`)
          .catch(err => {
            // If schedules not found, handle gracefully (new workflow)
            if (err.response && err.response.status === 404) {
              console.log('No schedules found - this is likely a new workflow');
              return { data: { schedules: [] } };
            }
            throw err;
          });
        
        setSchedules(response.data.schedules || []);
      } catch (error) {
        console.error('Error fetching schedules:', error);
        // Don't show error toast for 404s as they're expected for new workflows
        if (!error.response || error.response.status !== 404) {
          toast.error('Failed to load workflow schedules');
        }
      } finally {
        setLoading(false);
      }
    };
    
    if (workflowId) {
      fetchSchedules();
    }
  }, [workflowId]);

  // Add new schedule
  const handleAddSchedule = async (e) => {
    e.preventDefault();
    
    if (!newSchedule.name.trim()) {
      toast.error('Schedule name is required');
      return;
    }
    
    try {
      const response = await api.post(`/api/workflows/${workflowId}/schedules`, newSchedule);
      setSchedules([...schedules, response.data.schedule]);
      setNewSchedule({
        cron: '0 0 * * *',
        enabled: true,
        name: ''
      });
      setShowAddForm(false);
      toast.success('Schedule added successfully');
    } catch (error) {
      console.error('Error adding schedule:', error);
      toast.error('Failed to add schedule');
    }
  };

  // Delete schedule
  const handleDeleteSchedule = async (scheduleId) => {
    try {
      await api.delete(`/api/workflows/${workflowId}/schedules/${scheduleId}`);
      setSchedules(schedules.filter(s => s.id !== scheduleId));
      toast.success('Schedule deleted successfully');
    } catch (error) {
      console.error('Error deleting schedule:', error);
      toast.error('Failed to delete schedule');
    }
  };

  // Toggle schedule enabled/disabled
  const handleToggleSchedule = async (schedule) => {
    try {
      const updatedSchedule = { ...schedule, enabled: !schedule.enabled };
      await api.put(`/api/workflows/${workflowId}/schedules/${schedule.id}`, updatedSchedule);
      setSchedules(schedules.map(s => s.id === schedule.id ? { ...s, enabled: !s.enabled } : s));
      toast.success(`Schedule ${updatedSchedule.enabled ? 'enabled' : 'disabled'} successfully`);
    } catch (error) {
      console.error('Error updating schedule:', error);
      toast.error('Failed to update schedule');
    }
  };

  // Get human-readable description of cron expression
  const getCronDescription = (cron) => {
    const preset = cronPresets.find(p => p.value === cron);
    if (preset) {
      return preset.label;
    }
    
    // Basic descriptions for common patterns
    const parts = cron.split(' ');
    if (parts.length !== 5) return cron;
    
    const [minute, hour, dayOfMonth, month, dayOfWeek] = parts;
    
    if (minute === '*' && hour === '*' && dayOfMonth === '*' && month === '*' && dayOfWeek === '*') {
      return 'Every minute';
    }
    
    if (minute === '0' && hour === '*' && dayOfMonth === '*' && month === '*' && dayOfWeek === '*') {
      return 'Every hour';
    }
    
    if (minute === '0' && hour === '0' && dayOfMonth === '*' && month === '*' && dayOfWeek === '*') {
      return 'Every day at midnight';
    }
    
    return cron;
  };

  // Format next run time
  const formatNextRun = (nextRun) => {
    if (!nextRun) return 'Not scheduled';
    
    const date = new Date(nextRun);
    return date.toLocaleString();
  };

  return (
    <div className="bg-white shadow overflow-hidden sm:rounded-md">
      <div className="px-4 py-5 border-b border-gray-200 sm:px-6 flex justify-between items-center">
        <div>
          <h3 className="text-lg leading-6 font-medium text-gray-900">
            Scheduled Executions
          </h3>
          <p className="mt-1 text-sm text-gray-500">
            Set up automatic workflow execution on a schedule
          </p>
        </div>
        <button
          onClick={() => setShowAddForm(!showAddForm)}
          className="inline-flex items-center px-3 py-2 border border-transparent shadow-sm text-sm leading-4 font-medium rounded-md text-white bg-primary-600 hover:bg-primary-700 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-primary-500"
        >
          <PlusIcon className="-ml-0.5 mr-2 h-4 w-4" />
          Add Schedule
        </button>
      </div>
      
      {/* Add schedule form */}
      {showAddForm && (
        <div className="px-4 py-5 sm:p-6 border-b border-gray-200">
          <form onSubmit={handleAddSchedule}>
            <div className="grid grid-cols-1 gap-y-6 gap-x-4 sm:grid-cols-6">
              <div className="sm:col-span-3">
                <label htmlFor="schedule-name" className="block text-sm font-medium text-gray-700">
                  Schedule Name
                </label>
                <div className="mt-1">
                  <input
                    type="text"
                    id="schedule-name"
                    value={newSchedule.name}
                    onChange={(e) => setNewSchedule({ ...newSchedule, name: e.target.value })}
                    className="shadow-sm focus:ring-primary-500 focus:border-primary-500 block w-full sm:text-sm border-gray-300 rounded-md"
                    placeholder="Daily Backup"
                    required
                  />
                </div>
              </div>
              
              <div className="sm:col-span-3">
                <label htmlFor="cron-expression" className="block text-sm font-medium text-gray-700">
                  Cron Expression
                </label>
                <div className="mt-1">
                  <input
                    type="text"
                    id="cron-expression"
                    value={newSchedule.cron}
                    onChange={(e) => setNewSchedule({ ...newSchedule, cron: e.target.value })}
                    className="shadow-sm focus:ring-primary-500 focus:border-primary-500 block w-full sm:text-sm border-gray-300 rounded-md"
                    placeholder="0 0 * * *"
                    required
                  />
                </div>
              </div>
              
              <div className="sm:col-span-6">
                <label className="block text-sm font-medium text-gray-700">
                  Preset Schedules
                </label>
                <div className="mt-1 grid grid-cols-1 gap-2 sm:grid-cols-2 lg:grid-cols-3">
                  {cronPresets.map((preset) => (
                    <button
                      key={preset.value}
                      type="button"
                      onClick={() => setNewSchedule({ ...newSchedule, cron: preset.value })}
                      className="inline-flex items-center px-2.5 py-1.5 border border-gray-300 shadow-sm text-xs font-medium rounded text-gray-700 bg-white hover:bg-gray-50 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-primary-500"
                    >
                      {preset.label}
                    </button>
                  ))}
                </div>
                <p className="mt-2 text-sm text-gray-500">
                  Format: minute hour day-of-month month day-of-week
                </p>
              </div>
              
              <div className="sm:col-span-6">
                <div className="flex items-start">
                  <div className="flex items-center h-5">
                    <input
                      id="schedule-enabled"
                      type="checkbox"
                      checked={newSchedule.enabled}
                      onChange={(e) => setNewSchedule({ ...newSchedule, enabled: e.target.checked })}
                      className="focus:ring-primary-500 h-4 w-4 text-primary-600 border-gray-300 rounded"
                    />
                  </div>
                  <div className="ml-3 text-sm">
                    <label htmlFor="schedule-enabled" className="font-medium text-gray-700">
                      Enabled
                    </label>
                    <p className="text-gray-500">Enable this schedule immediately after creation</p>
                  </div>
                </div>
              </div>
            </div>
            
            <div className="mt-5 sm:mt-6 sm:flex sm:flex-row-reverse">
              <button
                type="submit"
                className="w-full inline-flex justify-center rounded-md border border-transparent shadow-sm px-4 py-2 bg-primary-600 text-base font-medium text-white hover:bg-primary-700 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-primary-500 sm:ml-3 sm:w-auto sm:text-sm"
              >
                Add Schedule
              </button>
              <button
                type="button"
                onClick={() => setShowAddForm(false)}
                className="mt-3 w-full inline-flex justify-center rounded-md border border-gray-300 shadow-sm px-4 py-2 bg-white text-base font-medium text-gray-700 hover:bg-gray-50 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-primary-500 sm:mt-0 sm:w-auto sm:text-sm"
              >
                Cancel
              </button>
            </div>
          </form>
        </div>
      )}
      
      {/* Schedules list */}
      {loading ? (
        <div className="px-4 py-5 sm:p-6">
          <div className="animate-pulse space-y-4">
            {[...Array(2)].map((_, i) => (
              <div key={i} className="h-16 bg-gray-200 rounded"></div>
            ))}
          </div>
        </div>
      ) : schedules.length > 0 ? (
        <ul className="divide-y divide-gray-200">
          {schedules.map((schedule) => (
            <li key={schedule.id} className="px-4 py-4 sm:px-6">
              <div className="flex items-center justify-between">
                <div className="flex items-center">
                  <div className={`flex-shrink-0 h-10 w-10 rounded-md flex items-center justify-center ${
                    schedule.enabled ? 'bg-green-100 text-green-600' : 'bg-gray-100 text-gray-600'
                  }`}>
                    <CalendarIcon className="h-6 w-6" />
                  </div>
                  <div className="ml-4">
                    <div className="flex items-center">
                      <div className="text-sm font-medium text-gray-900">{schedule.name}</div>
                      <span className={`ml-2 inline-flex items-center px-2.5 py-0.5 rounded-full text-xs font-medium ${
                        schedule.enabled ? 'bg-green-100 text-green-800' : 'bg-gray-100 text-gray-800'
                      }`}>
                        {schedule.enabled ? 'Enabled' : 'Disabled'}
                      </span>
                    </div>
                    <div className="text-sm text-gray-500">
                      {getCronDescription(schedule.cron)}
                    </div>
                    <div className="flex items-center text-xs text-gray-500 mt-1">
                      <ClockIcon className="h-4 w-4 mr-1" />
                      Next run: {formatNextRun(schedule.next_run)}
                    </div>
                  </div>
                </div>
                <div className="flex space-x-2">
                  <button
                    onClick={() => handleToggleSchedule(schedule)}
                    className={`inline-flex items-center p-1.5 border border-gray-300 shadow-sm text-xs font-medium rounded ${
                      schedule.enabled
                        ? 'text-red-700 hover:bg-red-50'
                        : 'text-green-700 hover:bg-green-50'
                    } bg-white focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-primary-500`}
                  >
                    {schedule.enabled ? 'Disable' : 'Enable'}
                  </button>
                  <button
                    onClick={() => handleDeleteSchedule(schedule.id)}
                    className="inline-flex items-center p-1.5 border border-gray-300 shadow-sm text-xs font-medium rounded text-gray-700 bg-white hover:bg-gray-50 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-primary-500"
                  >
                    <TrashIcon className="h-4 w-4 text-red-500" />
                  </button>
                </div>
              </div>
            </li>
          ))}
        </ul>
      ) : (
        <div className="px-4 py-5 sm:p-6 text-center">
          <CalendarIcon className="mx-auto h-12 w-12 text-gray-400" />
          <h3 className="mt-2 text-sm font-medium text-gray-900">No schedules</h3>
          <p className="mt-1 text-sm text-gray-500">
            Create a schedule to run this workflow automatically.
          </p>
          {!showAddForm && (
            <div className="mt-6">
              <button
                type="button"
                onClick={() => setShowAddForm(true)}
                className="inline-flex items-center px-4 py-2 border border-transparent shadow-sm text-sm font-medium rounded-md text-white bg-primary-600 hover:bg-primary-700 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-primary-500"
              >
                <PlusIcon className="-ml-1 mr-2 h-5 w-5" aria-hidden="true" />
                New Schedule
              </button>
            </div>
          )}
        </div>
      )}
    </div>
  );
};

export default CronScheduler;
