import React, { memo, useCallback } from 'react';
import { Handle, Position } from 'reactflow';

const getNodeColor = (type) => {
  switch (type) {
    case 'webhook':
      return 'border-blue-500';
    case 'http-request':
      return 'border-green-500';
    case 'function':
      return 'border-purple-500';
    case 'delay':
      return 'border-amber-500';
    case 'email':
      return 'border-red-500';
    case 'logger':
      return 'border-gray-500';
    default:
      return 'border-gray-300';
  }
};

const getNodeIcon = (type) => {
  switch (type) {
    case 'webhook':
      return (
        <svg xmlns="http://www.w3.org/2000/svg" className="h-5 w-5" fill="none" viewBox="0 0 24 24" stroke="currentColor">
          <path strokeLinecap="round" strokeLinejoin="round" strokeWidth={2} d="M8 16l2.879-2.879m0 0a3 3 0 104.243-4.242 3 3 0 00-4.243 4.242zM21 12a9 9 0 11-18 0 9 9 0 0118 0z" />
        </svg>
      );
    case 'http-request':
      return (
        <svg xmlns="http://www.w3.org/2000/svg" className="h-5 w-5" fill="none" viewBox="0 0 24 24" stroke="currentColor">
          <path strokeLinecap="round" strokeLinejoin="round" strokeWidth={2} d="M21 12a9 9 0 01-9 9m9-9a9 9 0 00-9-9m9 9H3m9 9a9 9 0 01-9-9m9 9c1.657 0 3-4.03 3-9s1.343-9-3-9m0 18c-1.657 0-3-4.03-3-9s1.343-9 3-9m-9 9a9 9 0 019-9" />
        </svg>
      );
    case 'function':
      return (
        <svg xmlns="http://www.w3.org/2000/svg" className="h-5 w-5" fill="none" viewBox="0 0 24 24" stroke="currentColor">
          <path strokeLinecap="round" strokeLinejoin="round" strokeWidth={2} d="M10 20l4-16m4 4l4 4-4 4M6 16l-4-4 4-4" />
        </svg>
      );
    case 'delay':
      return (
        <svg xmlns="http://www.w3.org/2000/svg" className="h-5 w-5" fill="none" viewBox="0 0 24 24" stroke="currentColor">
          <path strokeLinecap="round" strokeLinejoin="round" strokeWidth={2} d="M12 8v4l3 3m6-3a9 9 0 11-18 0 9 9 0 0118 0z" />
        </svg>
      );
    case 'email':
      return (
        <svg xmlns="http://www.w3.org/2000/svg" className="h-5 w-5" fill="none" viewBox="0 0 24 24" stroke="currentColor">
          <path strokeLinecap="round" strokeLinejoin="round" strokeWidth={2} d="M3 8l7.89 5.26a2 2 0 002.22 0L21 8M5 19h14a2 2 0 002-2V7a2 2 0 00-2-2H5a2 2 0 00-2 2v10a2 2 0 002 2z" />
        </svg>
      );
    case 'logger':
      return (
        <svg xmlns="http://www.w3.org/2000/svg" className="h-5 w-5" fill="none" viewBox="0 0 24 24" stroke="currentColor">
          <path strokeLinecap="round" strokeLinejoin="round" strokeWidth={2} d="M9 12h6m-6 4h6m2 5H7a2 2 0 01-2-2V5a2 2 0 012-2h5.586a1 1 0 01.707.293l5.414 5.414a1 1 0 01.293.707V19a2 2 0 01-2 2z" />
        </svg>
      );
    default:
      return (
        <svg xmlns="http://www.w3.org/2000/svg" className="h-5 w-5" fill="none" viewBox="0 0 24 24" stroke="currentColor">
          <path strokeLinecap="round" strokeLinejoin="round" strokeWidth={2} d="M9 5H7a2 2 0 00-2 2v12a2 2 0 002 2h10a2 2 0 002-2V7a2 2 0 00-2-2h-2M9 5a2 2 0 002 2h2a2 2 0 002-2M9 5a2 2 0 012-2h2a2 2 0 012 2" />
        </svg>
      );
  }
};

const CustomNode = ({ id, data, selected, onEdit, onDelete }) => {
  const nodeColor = getNodeColor(data.nodeType);
  const nodeIcon = getNodeIcon(data.nodeType);

  const handleEdit = useCallback((e) => {
    e?.stopPropagation?.();
    e?.preventDefault?.();
    if (onEdit) {
      onEdit(id);
    }
  }, [id, onEdit]);

  const handleDelete = useCallback((e) => {
    e?.stopPropagation?.();
    e?.preventDefault?.();
    if (onDelete) {
      onDelete(id);
    }
  }, [id, onDelete]);

  return (
    <div 
      className={`group relative ${selected ? 'ring-2 ring-primary-400' : ''} bg-white rounded-md shadow-md border-l-4 ${nodeColor} p-3 min-w-[180px] cursor-grab active:cursor-grabbing hover:shadow-lg transition-all duration-200`}
      style={{ pointerEvents: 'auto' }}
    >
      {/* Action buttons - always visible for debugging */}
      <div 
        className="absolute -top-2 -right-2 flex space-x-1 transition-all duration-200 z-50 opacity-100"
        style={{ pointerEvents: 'auto' }}
      >
        {/* Edit button */}
        <div
          onMouseDown={handleEdit}
          className="w-10 h-10 bg-blue-500 hover:bg-blue-600 text-white rounded-full flex items-center justify-center shadow-xl transition-all duration-200 hover:scale-105 border-2 border-white cursor-pointer select-none"
          style={{ 
            pointerEvents: 'auto',
            userSelect: 'none',
            position: 'relative',
            zIndex: 1000,
            touchAction: 'manipulation'
          }}
          title="Edit node"
          role="button"
          tabIndex={0}
        >
          <svg xmlns="http://www.w3.org/2000/svg" className="h-5 w-5 pointer-events-none" fill="none" viewBox="0 0 24 24" stroke="currentColor">
            <path strokeLinecap="round" strokeLinejoin="round" strokeWidth={2} d="M11 5H6a2 2 0 00-2 2v11a2 2 0 002 2h11a2 2 0 002-2v-5m-1.414-9.414a2 2 0 112.828 2.828L11.828 15H9v-2.828l8.586-8.586z" />
          </svg>
        </div>
        
        {/* Delete button */}
        <div
          onMouseDown={handleDelete}
          className="w-10 h-10 bg-red-500 hover:bg-red-600 text-white rounded-full flex items-center justify-center shadow-xl transition-all duration-200 hover:scale-105 border-2 border-white cursor-pointer select-none"
          style={{ 
            pointerEvents: 'auto',
            userSelect: 'none',
            position: 'relative',
            zIndex: 1000,
            touchAction: 'manipulation'
          }}
          title="Delete node"
          role="button"
          tabIndex={0}
        >
          <svg xmlns="http://www.w3.org/2000/svg" className="h-5 w-5 pointer-events-none" fill="none" viewBox="0 0 24 24" stroke="currentColor">
            <path strokeLinecap="round" strokeLinejoin="round" strokeWidth={2} d="M19 7l-.867 12.142A2 2 0 0116.138 21H7.862a2 2 0 01-1.995-1.858L5 7m5 4v6m4-6v6m1-10V4a1 1 0 00-1-1h-4a1 1 0 00-1 1v3M4 7h16" />
          </svg>
        </div>
      </div>

      {/* Input handle */}
      <Handle
        type="target"
        position={Position.Top}
        className="w-3 h-3 rounded-full bg-gray-400 border-2 border-white"
        style={{ pointerEvents: 'auto' }}
      />
      
      {/* Node content */}
      <div className="flex items-center" style={{ pointerEvents: 'none' }}>
        <div className="flex-shrink-0 text-gray-500">
          {nodeIcon}
        </div>
        <div className="ml-2 flex-1">
          <div className="text-sm font-medium text-gray-900 truncate">
            {data.label}
          </div>
          <div className="text-xs text-gray-500">
            {data.nodeType}
          </div>
        </div>
      </div>
      
      {/* Output handle */}
      <Handle
        type="source"
        position={Position.Bottom}
        className="w-3 h-3 rounded-full bg-gray-400 border-2 border-white"
        style={{ pointerEvents: 'auto' }}
      />
    </div>
  );
};

export default memo(CustomNode);
