import React, { useState, useEffect } from 'react';
import { XMarkIcon } from '@heroicons/react/24/outline';
import api from '../../services/api';

const NodeConfigPanel = ({ node, onConfigUpdate, onClose }) => {
  const [nodeMeta, setNodeMeta] = useState(null);
  const [config, setConfig] = useState({});
  const [nodeName, setNodeName] = useState('');
  const [loading, setLoading] = useState(true);

  useEffect(() => {
    const fetchNodeMeta = async () => {
      try {
        setLoading(true);
        const response = await api.get(`/api/nodes/${node.data.nodeType}/meta`);
        setNodeMeta(response.data);
        
        // Initialize config with existing values or defaults
        const initialConfig = { ...node.data.config };
        setConfig(initialConfig);
        
        // Set node name
        setNodeName(node.data.label);
      } catch (error) {
        console.error('Error fetching node metadata:', error);
      } finally {
        setLoading(false);
      }
    };

    if (node) {
      fetchNodeMeta();
    }
  }, [node]);

  // Handle config field change
  const handleConfigChange = (key, value) => {
    const updatedConfig = { ...config, [key]: value };
    setConfig(updatedConfig);
    onConfigUpdate(node.id, updatedConfig);
  };

  // Handle node name change
  const handleNameChange = (e) => {
    const newName = e.target.value;
    setNodeName(newName);
    
    // Update node data in parent component
    const updatedNode = {
      ...node,
      data: {
        ...node.data,
        label: newName
      }
    };
    
    // This will trigger a re-render of the node with the new name
    onConfigUpdate(node.id, node.data.config, updatedNode);
  };

  // Render form field based on schema type
  const renderFormField = (key, schema) => {
    const value = config[key] !== undefined ? config[key] : (schema.default || '');
    
    switch (schema.type) {
      case 'string':
        if (schema.enum) {
          return (
            <select
              id={key}
              value={value}
              onChange={(e) => handleConfigChange(key, e.target.value)}
              className="mt-1 block w-full pl-3 pr-10 py-2 text-base border-gray-300 focus:outline-none focus:ring-primary-500 focus:border-primary-500 sm:text-sm rounded-md"
            >
              {schema.enum.map((option) => (
                <option key={option} value={option}>
                  {option}
                </option>
              ))}
            </select>
          );
        }
        return (
          <input
            type="text"
            id={key}
            value={value}
            onChange={(e) => handleConfigChange(key, e.target.value)}
            placeholder={schema.description || ''}
            className="mt-1 block w-full border-gray-300 rounded-md shadow-sm focus:ring-primary-500 focus:border-primary-500 sm:text-sm"
          />
        );
      
      case 'number':
      case 'integer':
        return (
          <input
            type="number"
            id={key}
            value={value}
            onChange={(e) => handleConfigChange(key, parseFloat(e.target.value))}
            placeholder={schema.description || ''}
            min={schema.minimum}
            max={schema.maximum}
            step={schema.type === 'integer' ? 1 : 'any'}
            className="mt-1 block w-full border-gray-300 rounded-md shadow-sm focus:ring-primary-500 focus:border-primary-500 sm:text-sm"
          />
        );
      
      case 'boolean':
        return (
          <input
            type="checkbox"
            id={key}
            checked={!!value}
            onChange={(e) => handleConfigChange(key, e.target.checked)}
            className="h-4 w-4 text-primary-600 focus:ring-primary-500 border-gray-300 rounded"
          />
        );
      
      case 'object':
        return (
          <textarea
            id={key}
            value={typeof value === 'object' ? JSON.stringify(value, null, 2) : value}
            onChange={(e) => {
              try {
                const parsedValue = JSON.parse(e.target.value);
                handleConfigChange(key, parsedValue);
              } catch (error) {
                // Allow invalid JSON during typing
                handleConfigChange(key, e.target.value);
              }
            }}
            rows={5}
            placeholder={schema.description || ''}
            className="mt-1 block w-full border-gray-300 rounded-md shadow-sm focus:ring-primary-500 focus:border-primary-500 sm:text-sm font-mono"
          />
        );
      
      default:
        return (
          <input
            type="text"
            id={key}
            value={value}
            onChange={(e) => handleConfigChange(key, e.target.value)}
            placeholder={schema.description || ''}
            className="mt-1 block w-full border-gray-300 rounded-md shadow-sm focus:ring-primary-500 focus:border-primary-500 sm:text-sm"
          />
        );
    }
  };

  if (loading) {
    return (
      <div className="p-4">
        <div className="flex items-center justify-between mb-4">
          <h3 className="text-lg font-medium text-gray-900">Node Configuration</h3>
          <button
            onClick={onClose}
            className="rounded-md text-gray-400 hover:text-gray-500 focus:outline-none focus:ring-2 focus:ring-primary-500"
          >
            <XMarkIcon className="h-5 w-5" />
          </button>
        </div>
        <div className="animate-pulse space-y-4">
          <div className="h-8 bg-gray-200 rounded"></div>
          <div className="h-24 bg-gray-200 rounded"></div>
          <div className="h-8 bg-gray-200 rounded"></div>
        </div>
      </div>
    );
  }

  return (
    <div className="p-4">
      <div className="flex items-center justify-between mb-4">
        <h3 className="text-lg font-medium text-gray-900">Node Configuration</h3>
        <button
          onClick={onClose}
          className="rounded-md text-gray-400 hover:text-gray-500 focus:outline-none focus:ring-2 focus:ring-primary-500"
        >
          <XMarkIcon className="h-5 w-5" />
        </button>
      </div>
      
      <div className="space-y-6">
        {/* Node name */}
        <div>
          <label htmlFor="node-name" className="block text-sm font-medium text-gray-700">
            Node Name
          </label>
          <input
            type="text"
            id="node-name"
            value={nodeName}
            onChange={handleNameChange}
            className="mt-1 block w-full border-gray-300 rounded-md shadow-sm focus:ring-primary-500 focus:border-primary-500 sm:text-sm"
          />
        </div>
        
        {/* Node type info */}
        <div>
          <h4 className="text-sm font-medium text-gray-700">Node Type</h4>
          <p className="mt-1 text-sm text-gray-500">{node.data.nodeType}</p>
          {nodeMeta?.description && (
            <p className="mt-1 text-xs text-gray-500">{nodeMeta.description}</p>
          )}
        </div>
        
        {/* Configuration fields */}
        {nodeMeta?.config?.schema?.properties && (
          <div>
            <h4 className="text-sm font-medium text-gray-700 mb-3">Configuration</h4>
            <div className="space-y-4">
              {Object.entries(nodeMeta.config.schema.properties).map(([key, schema]) => (
                <div key={key}>
                  <label htmlFor={key} className="block text-sm font-medium text-gray-700">
                    {schema.title || key}
                    {schema.required && <span className="text-red-500">*</span>}
                  </label>
                  {schema.description && (
                    <p className="mt-1 text-xs text-gray-500">{schema.description}</p>
                  )}
                  <div className="mt-1">
                    {renderFormField(key, schema)}
                  </div>
                </div>
              ))}
            </div>
          </div>
        )}
        
        {/* Input/Output info */}
        <div className="pt-4 border-t border-gray-200">
          <div className="grid grid-cols-2 gap-4">
            <div>
              <h4 className="text-sm font-medium text-gray-700 mb-2">Inputs</h4>
              {nodeMeta?.inputs?.length > 0 ? (
                <ul className="text-xs text-gray-500 space-y-1">
                  {nodeMeta.inputs.map((input, index) => (
                    <li key={index} className="flex items-center">
                      <span className="w-2 h-2 bg-gray-400 rounded-full mr-2"></span>
                      {input.name || 'Default'}
                    </li>
                  ))}
                </ul>
              ) : (
                <p className="text-xs text-gray-500">No inputs</p>
              )}
            </div>
            <div>
              <h4 className="text-sm font-medium text-gray-700 mb-2">Outputs</h4>
              {nodeMeta?.outputs?.length > 0 ? (
                <ul className="text-xs text-gray-500 space-y-1">
                  {nodeMeta.outputs.map((output, index) => (
                    <li key={index} className="flex items-center">
                      <span className="w-2 h-2 bg-gray-400 rounded-full mr-2"></span>
                      {output.name || 'Default'}
                    </li>
                  ))}
                </ul>
              ) : (
                <p className="text-xs text-gray-500">No outputs</p>
              )}
            </div>
          </div>
        </div>
      </div>
    </div>
  );
};

export default NodeConfigPanel;
