import React, { useState, useEffect } from 'react';
import { toast } from 'react-toastify';
import api from '../../services/api';
import Modal from '../common/Modal';
import {
  ClockIcon,
  ArrowPathIcon,
  EyeIcon,
  CheckIcon
} from '@heroicons/react/24/outline';

const VersionHistory = ({ workflowId, currentVersion, onRestoreVersion }) => {
  const [versions, setVersions] = useState([]);
  const [loading, setLoading] = useState(true);
  const [selectedVersion, setSelectedVersion] = useState(null);
  const [showVersionModal, setShowVersionModal] = useState(false);

  // Fetch version history
  useEffect(() => {
    const fetchVersionHistory = async () => {
      try {
        setLoading(true);
        const response = await api.get(`/api/workflows/${workflowId}/versions`)
          .catch(err => {
            // If versions not found, handle gracefully (new workflow)
            if (err.response && err.response.status === 404) {
              console.log('No version history found - this is likely a new workflow');
              return { data: { versions: [] } };
            }
            throw err;
          });
        
        setVersions(response.data.versions || []);
      } catch (error) {
        console.error('Error fetching version history:', error);
        // Don't show error toast for 404s as they're expected for new workflows
        if (!error.response || error.response.status !== 404) {
          toast.error('Failed to load workflow versions');
        }
      } finally {
        setLoading(false);
      }
    };
    
    if (workflowId) {
      fetchVersionHistory();
    }
  }, [workflowId, currentVersion]);

  // Format timestamp
  const formatTimestamp = (timestamp) => {
    if (!timestamp) return '';
    const date = new Date(timestamp);
    return date.toLocaleString();
  };

  // View version details
  const viewVersion = (version) => {
    setSelectedVersion(version);
    setShowVersionModal(true);
  };

  // Restore version
  const handleRestoreVersion = async (version) => {
    try {
      await api.post(`/api/workflows/${workflowId}/versions/${version.version}/restore`);
      toast.success(`Restored workflow to version ${version.version}`);
      onRestoreVersion(version.version);
      setShowVersionModal(false);
    } catch (error) {
      console.error('Error restoring version:', error);
      toast.error('Failed to restore workflow version');
    }
  };

  // Render JSON diff
  const renderDiff = (current, previous) => {
    if (!current || !previous) return null;
    
    // This is a simple visual diff, a more sophisticated diff library could be used
    try {
      const currentObj = typeof current === 'string' ? JSON.parse(current) : current;
      const previousObj = typeof previous === 'string' ? JSON.parse(previous) : previous;
      
      // Compare nodes count
      const currentNodesCount = currentObj.nodes?.length || 0;
      const previousNodesCount = previousObj.nodes?.length || 0;
      
      // Compare edges count
      const currentEdgesCount = currentObj.edges?.length || 0;
      const previousEdgesCount = previousObj.edges?.length || 0;
      
      return (
        <div className="space-y-2 text-sm">
          <div className="flex justify-between">
            <span>Nodes:</span>
            <span className={currentNodesCount !== previousNodesCount ? 'font-medium text-amber-600' : ''}>
              {currentNodesCount} {currentNodesCount !== previousNodesCount && `(was ${previousNodesCount})`}
            </span>
          </div>
          <div className="flex justify-between">
            <span>Edges:</span>
            <span className={currentEdgesCount !== previousEdgesCount ? 'font-medium text-amber-600' : ''}>
              {currentEdgesCount} {currentEdgesCount !== previousEdgesCount && `(was ${previousEdgesCount})`}
            </span>
          </div>
        </div>
      );
    } catch (error) {
      console.error('Error parsing JSON for diff:', error);
      return <div className="text-red-500 text-sm">Error comparing versions</div>;
    }
  };

  return (
    <div className="bg-white shadow overflow-hidden sm:rounded-md">
      <div className="px-4 py-5 border-b border-gray-200 sm:px-6">
        <h3 className="text-lg leading-6 font-medium text-gray-900">
          Version History
        </h3>
        <p className="mt-1 text-sm text-gray-500">
          View and restore previous versions of this workflow
        </p>
      </div>
      
      {loading ? (
        <div className="px-4 py-5 sm:p-6">
          <div className="animate-pulse space-y-4">
            {[...Array(3)].map((_, i) => (
              <div key={i} className="h-16 bg-gray-200 rounded"></div>
            ))}
          </div>
        </div>
      ) : versions.length > 0 ? (
        <ul className="divide-y divide-gray-200">
          {versions.map((version) => (
            <li key={version.version} className="px-4 py-4 sm:px-6">
              <div className="flex items-center justify-between">
                <div className="flex items-center">
                  <div className={`flex-shrink-0 h-10 w-10 rounded-md flex items-center justify-center ${
                    version.version === currentVersion ? 'bg-green-100 text-green-600' : 'bg-gray-100 text-gray-600'
                  }`}>
                    {version.version === currentVersion ? (
                      <CheckIcon className="h-6 w-6" />
                    ) : (
                      <ClockIcon className="h-6 w-6" />
                    )}
                  </div>
                  <div className="ml-4">
                    <div className="flex items-center">
                      <div className="text-sm font-medium text-gray-900">
                        Version {version.version}
                      </div>
                      {version.version === currentVersion && (
                        <span className="ml-2 inline-flex items-center px-2.5 py-0.5 rounded-full text-xs font-medium bg-green-100 text-green-800">
                          Current
                        </span>
                      )}
                    </div>
                    <div className="text-sm text-gray-500">
                      Created: {formatTimestamp(version.created_at)}
                    </div>
                    {version.comment && (
                      <div className="text-sm text-gray-700 mt-1">
                        "{version.comment}"
                      </div>
                    )}
                  </div>
                </div>
                <div className="flex space-x-2">
                  <button
                    onClick={() => viewVersion(version)}
                    className="inline-flex items-center p-1.5 border border-gray-300 shadow-sm text-xs font-medium rounded text-gray-700 bg-white hover:bg-gray-50 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-primary-500"
                  >
                    <EyeIcon className="h-4 w-4" />
                  </button>
                  {version.version !== currentVersion && (
                    <button
                      onClick={() => handleRestoreVersion(version)}
                      className="inline-flex items-center p-1.5 border border-gray-300 shadow-sm text-xs font-medium rounded text-gray-700 bg-white hover:bg-gray-50 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-primary-500"
                    >
                      <ArrowPathIcon className="h-4 w-4 text-amber-500" />
                    </button>
                  )}
                </div>
              </div>
              
              {/* Show diff with previous version if available */}
              {version.version > 1 && versions.find(v => v.version === version.version - 1) && (
                <div className="mt-2 ml-14">
                  <div className="text-xs text-gray-500 mb-1">Changes from previous version:</div>
                  {renderDiff(
                    version.workflow_data,
                    versions.find(v => v.version === version.version - 1)?.workflow_data
                  )}
                </div>
              )}
            </li>
          ))}
        </ul>
      ) : (
        <div className="px-4 py-5 sm:p-6 text-center">
          <ClockIcon className="mx-auto h-12 w-12 text-gray-400" />
          <h3 className="mt-2 text-sm font-medium text-gray-900">No version history</h3>
          <p className="mt-1 text-sm text-gray-500">
            Save changes to your workflow to create new versions.
          </p>
        </div>
      )}
      
      {/* Version details modal */}
      {selectedVersion && (
        <Modal
          isOpen={showVersionModal}
          onClose={() => setShowVersionModal(false)}
          title={`Version ${selectedVersion.version} Details`}
          size="lg"
          footer={
            <>
              {selectedVersion.version !== currentVersion && (
                <button
                  type="button"
                  onClick={() => handleRestoreVersion(selectedVersion)}
                  className="inline-flex justify-center rounded-md border border-transparent shadow-sm px-4 py-2 bg-amber-600 text-base font-medium text-white hover:bg-amber-700 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-amber-500 sm:ml-3 sm:w-auto sm:text-sm"
                >
                  Restore This Version
                </button>
              )}
              <button
                type="button"
                onClick={() => setShowVersionModal(false)}
                className="mt-3 w-full inline-flex justify-center rounded-md border border-gray-300 shadow-sm px-4 py-2 bg-white text-base font-medium text-gray-700 hover:bg-gray-50 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-primary-500 sm:mt-0 sm:ml-3 sm:w-auto sm:text-sm"
              >
                Close
              </button>
            </>
          }
        >
          <div className="space-y-6">
            <div>
              <h4 className="text-sm font-medium text-gray-900">Version Information</h4>
              <dl className="mt-2 grid grid-cols-1 gap-x-4 gap-y-4 sm:grid-cols-2">
                <div>
                  <dt className="text-sm font-medium text-gray-500">Version Number</dt>
                  <dd className="mt-1 text-sm text-gray-900">{selectedVersion.version}</dd>
                </div>
                <div>
                  <dt className="text-sm font-medium text-gray-500">Created At</dt>
                  <dd className="mt-1 text-sm text-gray-900">{formatTimestamp(selectedVersion.created_at)}</dd>
                </div>
                {selectedVersion.comment && (
                  <div className="sm:col-span-2">
                    <dt className="text-sm font-medium text-gray-500">Comment</dt>
                    <dd className="mt-1 text-sm text-gray-900">{selectedVersion.comment}</dd>
                  </div>
                )}
              </dl>
            </div>
            
            <div>
              <h4 className="text-sm font-medium text-gray-900">Workflow Data</h4>
              <div className="mt-2 bg-gray-50 p-4 rounded-md overflow-auto max-h-96">
                <pre className="text-xs text-gray-800">
                  {JSON.stringify(
                    typeof selectedVersion.workflow_data === 'string' 
                      ? JSON.parse(selectedVersion.workflow_data) 
                      : selectedVersion.workflow_data,
                    null, 2
                  )}
                </pre>
              </div>
            </div>
          </div>
        </Modal>
      )}
    </div>
  );
};

export default VersionHistory;
