import React, { useState, useEffect } from 'react';
import { toast } from 'react-toastify';
import { ClipboardDocumentIcon, CheckIcon } from '@heroicons/react/24/outline';
import nodeService from '../../services/nodes';

const WebhookManager = ({ workflowId, nodes }) => {
  const [webhooks, setWebhooks] = useState({});
  const [loading, setLoading] = useState(true);
  const [copiedWebhook, setCopiedWebhook] = useState(null);

  // Get webhook nodes from the workflow
  const webhookNodes = nodes.filter(node => node.data.nodeType === 'webhook');

  // Fetch webhook URLs for all webhook nodes
  useEffect(() => {
    const fetchWebhookUrls = async () => {
      if (webhookNodes.length === 0) {
        setLoading(false);
        return;
      }

      try {
        setLoading(true);
        const webhookData = {};
        
        // Fetch webhook URL for each webhook node
        for (const node of webhookNodes) {
          try {
            const response = await nodeService.getWebhookUrl(workflowId, node.id);
            webhookData[node.id] = response.data.webhookUrl;
          } catch (error) {
            console.error(`Error fetching webhook URL for node ${node.id}:`, error);
            webhookData[node.id] = null;
          }
        }
        
        setWebhooks(webhookData);
      } catch (error) {
        console.error('Error fetching webhook URLs:', error);
        toast.error('Failed to load webhook URLs');
      } finally {
        setLoading(false);
      }
    };

    if (workflowId && webhookNodes.length > 0) {
      fetchWebhookUrls();
    } else {
      setLoading(false);
    }
  }, [workflowId, webhookNodes]);

  // Copy webhook URL to clipboard
  const copyToClipboard = (nodeId, url) => {
    navigator.clipboard.writeText(url).then(
      () => {
        setCopiedWebhook(nodeId);
        setTimeout(() => setCopiedWebhook(null), 2000);
        toast.success('Webhook URL copied to clipboard');
      },
      () => {
        toast.error('Failed to copy webhook URL');
      }
    );
  };

  if (webhookNodes.length === 0) {
    return null;
  }

  return (
    <div className="mt-6 bg-white shadow overflow-hidden sm:rounded-md">
      <div className="px-4 py-5 border-b border-gray-200 sm:px-6">
        <h3 className="text-lg leading-6 font-medium text-gray-900">
          Webhook Endpoints
        </h3>
        <p className="mt-1 text-sm text-gray-500">
          Use these URLs to trigger your workflow from external services
        </p>
      </div>
      
      <ul className="divide-y divide-gray-200">
        {webhookNodes.map((node) => (
          <li key={node.id} className="px-4 py-4 sm:px-6">
            <div className="flex items-center justify-between">
              <div className="flex items-center">
                <div className="flex-shrink-0 h-10 w-10 bg-blue-100 text-blue-600 rounded-md flex items-center justify-center">
                  <svg xmlns="http://www.w3.org/2000/svg" className="h-6 w-6" fill="none" viewBox="0 0 24 24" stroke="currentColor">
                    <path strokeLinecap="round" strokeLinejoin="round" strokeWidth={2} d="M8 16l2.879-2.879m0 0a3 3 0 104.243-4.242 3 3 0 00-4.243 4.242zM21 12a9 9 0 11-18 0 9 9 0 0118 0z" />
                  </svg>
                </div>
                <div className="ml-4">
                  <div className="text-sm font-medium text-gray-900">{node.data.label}</div>
                  <div className="text-xs text-gray-500">Node ID: {node.id}</div>
                </div>
              </div>
            </div>
            
            <div className="mt-2">
              {loading ? (
                <div className="animate-pulse h-8 bg-gray-200 rounded w-full"></div>
              ) : webhooks[node.id] ? (
                <div className="flex items-center mt-1 relative rounded-md shadow-sm">
                  <input
                    type="text"
                    readOnly
                    value={webhooks[node.id]}
                    className="focus:ring-primary-500 focus:border-primary-500 block w-full pr-10 sm:text-sm border-gray-300 rounded-md bg-gray-50"
                  />
                  <div className="absolute inset-y-0 right-0 pr-3 flex items-center">
                    <button
                      type="button"
                      onClick={() => copyToClipboard(node.id, webhooks[node.id])}
                      className="text-gray-400 hover:text-gray-500 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-primary-500"
                    >
                      {copiedWebhook === node.id ? (
                        <CheckIcon className="h-5 w-5 text-green-500" aria-hidden="true" />
                      ) : (
                        <ClipboardDocumentIcon className="h-5 w-5" aria-hidden="true" />
                      )}
                    </button>
                  </div>
                </div>
              ) : (
                <div className="text-sm text-red-500">
                  Failed to load webhook URL. Please save the workflow first.
                </div>
              )}
            </div>
          </li>
        ))}
      </ul>
    </div>
  );
};

export default WebhookManager;
