import React, { useState } from 'react';
import { toast } from 'react-toastify';
import Modal from '../common/Modal';
import NodeTester from './NodeTester';
import ExecutionResults from '../execution/ExecutionResults';
import {
  PlayIcon,
  PencilIcon,
  DocumentDuplicateIcon,
  TrashIcon,
  BeakerIcon
} from '@heroicons/react/24/outline';
import api from '../../services/api';

const WorkflowEditorActions = ({ 
  workflowId, 
  selectedNode, 
  onDuplicateNode, 
  onDeleteNode,
  onExecuteWorkflow
}) => {
  const [showNodeTester, setShowNodeTester] = useState(false);
  const [showExecutionResults, setShowExecutionResults] = useState(false);
  const [latestExecutionId, setLatestExecutionId] = useState(null);

  // Execute the workflow
  const handleExecute = async () => {
    try {
      const response = await api.post(`/api/workflows/${workflowId}/execute`);
      setLatestExecutionId(response.data.execution.id);
      setShowExecutionResults(true);
      toast.success('Workflow execution started');
      
      if (onExecuteWorkflow) {
        onExecuteWorkflow(response.data.execution.id);
      }
    } catch (error) {
      console.error('Error executing workflow:', error);
      toast.error('Failed to execute workflow');
    }
  };

  // Open node tester
  const handleTestNode = () => {
    if (!selectedNode) {
      toast.warning('Please select a node to test');
      return;
    }
    setShowNodeTester(true);
  };

  // Duplicate selected node
  const handleDuplicateNode = () => {
    if (!selectedNode) {
      toast.warning('Please select a node to duplicate');
      return;
    }
    
    if (onDuplicateNode) {
      onDuplicateNode(selectedNode);
      toast.success('Node duplicated');
    }
  };

  // Delete selected node
  const handleDeleteNode = () => {
    if (!selectedNode) {
      toast.warning('Please select a node to delete');
      return;
    }
    
    if (onDeleteNode) {
      onDeleteNode(selectedNode);
      toast.success('Node deleted');
    }
  };

  return (
    <>
      <div className="flex space-x-2">
        {/* Execute workflow button */}
        <button
          onClick={handleExecute}
          disabled={!workflowId || workflowId === 'new'}
          className="inline-flex items-center px-3 py-2 border border-transparent text-sm leading-4 font-medium rounded-md shadow-sm text-white bg-green-600 hover:bg-green-700 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-green-500 disabled:opacity-50"
          title="Execute workflow"
        >
          <PlayIcon className="h-4 w-4 mr-1" />
          Execute
        </button>
        
        {/* Node actions */}
        <div className="border-l border-gray-300 pl-2 flex space-x-1">
          {/* Test node button */}
          <button
            onClick={handleTestNode}
            disabled={!selectedNode}
            className="inline-flex items-center p-2 border border-gray-300 rounded-md shadow-sm text-sm font-medium text-gray-700 bg-white hover:bg-gray-50 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-primary-500 disabled:opacity-50"
            title="Test selected node"
          >
            <BeakerIcon className="h-4 w-4" />
          </button>
          
          {/* Duplicate node button */}
          <button
            onClick={handleDuplicateNode}
            disabled={!selectedNode}
            className="inline-flex items-center p-2 border border-gray-300 rounded-md shadow-sm text-sm font-medium text-gray-700 bg-white hover:bg-gray-50 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-primary-500 disabled:opacity-50"
            title="Duplicate selected node"
          >
            <DocumentDuplicateIcon className="h-4 w-4" />
          </button>
          
          {/* Delete node button */}
          <button
            onClick={handleDeleteNode}
            disabled={!selectedNode}
            className="inline-flex items-center p-2 border border-gray-300 rounded-md shadow-sm text-sm font-medium text-gray-700 bg-white hover:bg-gray-50 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-primary-500 disabled:opacity-50"
            title="Delete selected node"
          >
            <TrashIcon className="h-4 w-4 text-red-500" />
          </button>
        </div>
      </div>
      
      {/* Node tester modal */}
      {selectedNode && (
        <Modal
          isOpen={showNodeTester}
          onClose={() => setShowNodeTester(false)}
          size="lg"
        >
          <NodeTester
            node={selectedNode}
            onClose={() => setShowNodeTester(false)}
          />
        </Modal>
      )}
      
      {/* Execution results modal */}
      {latestExecutionId && (
        <Modal
          isOpen={showExecutionResults}
          onClose={() => setShowExecutionResults(false)}
          size="lg"
        >
          <ExecutionResults
            workflowId={workflowId}
            executionId={latestExecutionId}
            onClose={() => setShowExecutionResults(false)}
          />
        </Modal>
      )}
    </>
  );
};

export default WorkflowEditorActions;
