import React, { useState } from 'react';
import { useNavigate } from 'react-router-dom';
import { toast } from 'react-toastify';
import automationTemplates from '../templates/templates';
import api from '../services/api';

const TemplatesPage = () => {
  const navigate = useNavigate();
  const [selectedCategory, setSelectedCategory] = useState('all');
  const [searchQuery, setSearchQuery] = useState('');
  
  // Extract unique tags from all templates
  const allTags = [...new Set(automationTemplates.flatMap(template => template.tags))];
  
  // Filter templates based on category and search query
  const filteredTemplates = automationTemplates.filter(template => {
    const matchesCategory = selectedCategory === 'all' || template.tags.includes(selectedCategory);
    const matchesSearch = searchQuery === '' || 
      template.name.toLowerCase().includes(searchQuery.toLowerCase()) ||
      template.description.toLowerCase().includes(searchQuery.toLowerCase()) ||
      template.tags.some(tag => tag.toLowerCase().includes(searchQuery.toLowerCase()));
    
    return matchesCategory && matchesSearch;
  });
  
  // Create a new workflow from template
  const createFromTemplate = async (template) => {
    try {
      console.log('Creating workflow from template:', template.name);
      
      // Create a new workflow with template data
      // Make sure we're sending proper data structures
      const requestData = {
        name: template.name,
        // Ensure nodes and connections are arrays
        nodes: Array.isArray(template.nodes) ? template.nodes : [],
        connections: Array.isArray(template.connections) ? template.connections : []
      };
      
      console.log('Sending request data:', requestData);
      
      const response = await api.post('/api/workflows', requestData);
      
      console.log('Response from server:', response.data);
      
      const newWorkflowId = response.data.workflow.id;
      toast.success(`Workflow "${template.name}" created successfully!`);
      
      // Wait a moment before navigating to ensure the backend has processed the workflow
      setTimeout(() => {
        // Navigate to the workflow editor
        navigate(`/workflows/${newWorkflowId}`);
      }, 500);
    } catch (error) {
      console.error('Error creating workflow from template:', error);
      console.error('Error response:', error.response?.data);
      toast.error(`Failed to create workflow: ${error.message}`);
    }
  };
  
  return (
    <div className="container mx-auto px-4 py-8">
      <h1 className="text-3xl font-bold mb-8">Automation Templates</h1>
      
      {/* Search and Filter */}
      <div className="flex flex-col md:flex-row justify-between mb-8 gap-4">
        <div className="flex-1">
          <input
            type="text"
            placeholder="Search templates..."
            className="w-full px-4 py-2 border border-gray-300 rounded-md focus:outline-none focus:ring-2 focus:ring-primary-500"
            value={searchQuery}
            onChange={(e) => setSearchQuery(e.target.value)}
          />
        </div>
        <div className="flex-shrink-0">
          <select
            className="px-4 py-2 border border-gray-300 rounded-md focus:outline-none focus:ring-2 focus:ring-primary-500"
            value={selectedCategory}
            onChange={(e) => setSelectedCategory(e.target.value)}
          >
            <option value="all">All Categories</option>
            {allTags.map(tag => (
              <option key={tag} value={tag}>{tag}</option>
            ))}
          </select>
        </div>
      </div>
      
      {/* Templates Grid */}
      <div className="grid grid-cols-1 md:grid-cols-2 lg:grid-cols-3 gap-6">
        {filteredTemplates.map(template => (
          <div 
            key={template.id}
            className="bg-white rounded-lg shadow-md overflow-hidden border border-gray-200 hover:shadow-lg transition-shadow"
          >
            <div className="p-6">
              <h2 className="text-xl font-semibold mb-2">{template.name}</h2>
              <p className="text-gray-600 mb-4">{template.description}</p>
              
              <div className="flex flex-wrap gap-2 mb-4">
                {template.tags.map(tag => (
                  <span 
                    key={`${template.id}-${tag}`}
                    className="bg-gray-100 text-gray-600 px-2 py-1 rounded-md text-sm"
                  >
                    {tag}
                  </span>
                ))}
              </div>
              
              <div className="flex justify-between items-center">
                <span className="text-sm text-gray-500">
                  {template.nodes.length} nodes
                </span>
                <button
                  onClick={() => createFromTemplate(template)}
                  className="px-4 py-2 bg-primary-600 text-white rounded-md hover:bg-primary-700 focus:outline-none focus:ring-2 focus:ring-primary-500 focus:ring-offset-2"
                >
                  Use Template
                </button>
              </div>
            </div>
          </div>
        ))}
      </div>
      
      {filteredTemplates.length === 0 && (
        <div className="text-center py-12">
          <p className="text-gray-500 text-xl">No templates found matching your criteria.</p>
        </div>
      )}
    </div>
  );
};

export default TemplatesPage;
