import api from './api';

/**
 * Service for workflow-related API calls
 */
const workflowService = {
  /**
   * Get all workflows for the current user
   * @returns {Promise} Promise resolving to workflow data
   */
  getWorkflows: async () => {
    return api.get('/api/workflows');
  },

  /**
   * Get a specific workflow by ID
   * @param {string} id Workflow ID
   * @returns {Promise} Promise resolving to workflow data
   */
  getWorkflow: async (id) => {
    return api.get(`/api/workflows/${id}`);
  },

  /**
   * Create a new workflow
   * @param {Object} workflow Workflow data
   * @returns {Promise} Promise resolving to created workflow
   */
  createWorkflow: async (workflow) => {
    return api.post('/api/workflows', workflow);
  },

  /**
   * Update an existing workflow
   * @param {string} id Workflow ID
   * @param {Object} workflow Updated workflow data
   * @returns {Promise} Promise resolving to updated workflow
   */
  updateWorkflow: async (id, workflow) => {
    return api.put(`/api/workflows/${id}`, workflow);
  },

  /**
   * Delete a workflow
   * @param {string} id Workflow ID
   * @returns {Promise} Promise resolving on successful deletion
   */
  deleteWorkflow: async (id) => {
    return api.delete(`/api/workflows/${id}`);
  },

  /**
   * Execute a workflow
   * @param {string} id Workflow ID
   * @param {Object} input Optional input data for the workflow
   * @returns {Promise} Promise resolving to execution data
   */
  executeWorkflow: async (id, input = {}) => {
    return api.post(`/api/workflows/${id}/execute`, { input });
  },

  /**
   * Get execution history for a workflow
   * @param {string} id Workflow ID
   * @param {Object} params Query parameters (limit, offset)
   * @returns {Promise} Promise resolving to execution history
   */
  getExecutionHistory: async (id, params = {}) => {
    return api.get(`/api/workflows/${id}/executions`, { params });
  },

  /**
   * Get details of a specific execution
   * @param {string} workflowId Workflow ID
   * @param {string} executionId Execution ID
   * @returns {Promise} Promise resolving to execution details
   */
  getExecution: async (workflowId, executionId) => {
    return api.get(`/api/workflows/${workflowId}/executions/${executionId}`);
  },

  /**
   * Get logs for a specific execution
   * @param {string} workflowId Workflow ID
   * @param {string} executionId Execution ID
   * @returns {Promise} Promise resolving to execution logs
   */
  getExecutionLogs: async (workflowId, executionId) => {
    return api.get(`/api/workflows/${workflowId}/executions/${executionId}/logs`);
  }
};

export default workflowService;
